(function() {

// class Fish
var Fish = function() {};
Fish.typeName = 'Fish';
Fish.prototype.domElement = null;
Fish.prototype.posX = 0;
Fish.prototype.posY = 0;
Fish.prototype.posZ = 0;
Fish.prototype.counter = 0;
Fish.prototype.velX = 0;
Fish.prototype.velY = 0;
Fish.prototype.velZ = 0;
Fish.prototype.size = 1;
Fish.prototype.gravity = 0;
Fish.prototype.enabled = true;

Fish.prototype.ctor$1 = function(imageSRC, imageWidth, imageHeight)
{
	this.domElement = document.createElement("div");
	this.domElement.style.background = "url(" + imageSRC + ") transparent";
	this.domElement.style.position = "absolute";
	this.domElement.style.display = "block";
	this.domElement.style.width = imageWidth.toString() + "px";
	this.domElement.style.height = imageHeight.toString() + "px";
	this.domElement.style.webkitTransformOrigin = ((imageWidth/2)|0).toString() + "px " + ((imageHeight/2)|0).toString() + "px";
	this.domElement.style.pointerEvents = "auto";
	return this;
};

Fish.prototype.init = function(posx, posy, posz)
{
	this.posX = posx;
	this.posY = posy;
	this.posZ = posz;
	this.enabled = true;
	this.domElement.style.visibility = "visible";
	this.velX = Utils.randomRange(-1, 1);
	this.velY = Utils.randomRange(-1, -2);
	this.velZ = Utils.randomRange(-1, 1);
	this.size = 1;
	this.gravity = -0.05;
};

Fish.prototype.update = function()
{
	if (!this.enabled)
		return true;
	this.velY += this.gravity;
	this.posX += this.velX;
	this.posY += this.velY;
	this.posZ += this.velZ;
	if (this.posY<-200)
	{
		this.domElement.style.visibility = "hidden";
		this.enabled = false;
		return false;
	}
	this.counter++;
	this.rotate(2, false);
	this.render();
	return true;
};

Fish.prototype.render = function()
{
	var sx=Math.sin(this.counter*0.4)*0.04 + this.size;
	var sy=Math.sin(Math.PI + this.counter*0.4)*0.04 + this.size;
	this.domElement.style.webkitTransform = "translate3d(" + this.posX.toString() + "px, " + this.posY.toString() + "px, " + this.posZ.toString() + "px) scale(" + sx.toString() + "," + sy.toString() + ") rotate(" + (Math.sin(this.counter*0.05)*20).toString() + "deg)";
};

Fish.prototype.rotate = function(angle, useRadians)
{
	var cosRY=Math.cos(angle*(useRadians ? 1 : Fish.TO_RADIANS));
	var sinRY=Math.sin(angle*(useRadians ? 1 : Fish.TO_RADIANS));
	var tempx=this.posX - HALF_WIDTH;
	this.posX = (tempx*cosRY) - (this.posZ*sinRY) + HALF_WIDTH;
	this.posZ = (tempx*sinRY) + (this.posZ*cosRY);
	tempx = this.velX;
	this.velX = (tempx*cosRY) - (this.velZ*sinRY);
	this.velZ = (tempx*sinRY) + (this.velZ*cosRY);
};

Fish.prototype.explode = function(emitter)
{
	emitter.makeExplosion(this.posX, this.posY, this.posZ);
	this.domElement.style.visibility = "hidden";
	this.enabled = false;
};
Fish.TO_RADIANS = 0;

Fish.$cctor = function()
{
	this.TO_RADIANS = Math.PI/180;
};

// class Emitter
var Emitter = function()
{
	this._particles = [];
	this._spareParticles = [];
};
Emitter.typeName = 'Emitter';
Emitter.prototype.domContainer = null;
Emitter.prototype._particles = null;
Emitter.prototype._spareParticles = null;

Emitter.prototype.ctor$1 = function(domContainer)
{
	Emitter.call(this);
	this.domContainer = domContainer;
	return this;
};

Emitter.prototype.update = function()
{
	for (var i=0; i<this._particles.length; i++)
	{
		var particle=this._particles[i];
		if (!particle.update())
		{
			this._spareParticles.push(particle);
		}
	}
};

Emitter.prototype.makeExplosion = function(xpos, ypos, zpos)
{
	for (var i=0; i<30; i++)
	{
		var particle=this.makeParticle();
		particle.init(xpos, ypos, zpos);
		particle.update();
	}
};

Emitter.prototype.makeParticle = function()
{
	var particle;
	if (this._spareParticles.length>0)
	{
		particle = this._spareParticles.shift();
	}
	else
	{
		particle = (new Particle().ctor$1(this.domContainer));
		this._particles.push(particle);
	}
	return particle;
};

// class Particle
var Particle = function() {};
Particle.typeName = 'Particle';
Particle.prototype.domElement = null;
Particle.prototype.x = 0;
Particle.prototype.y = 0;
Particle.prototype.z = 0;
Particle.prototype.xVel = 0;
Particle.prototype.yVel = 0;
Particle.prototype.zVel = 0;
Particle.prototype.size = 1;
Particle.prototype.enabled = true;

Particle.prototype.ctor$1 = function(domContainer)
{
	this.domElement = document.createElement("div");
	this.domElement.style.background = "url(img/orangeParticle.png) transparent";
	this.domElement.style.position = "absolute";
	this.domElement.style.display = "block";
	this.domElement.style.width = "32px";
	this.domElement.style.height = "32px";
	this.domElement.style.webkitTransformOrigin = "16px 16px";
	domContainer.appendChild(this.domElement);
	return this;
};

Particle.prototype.init = function(xpos, ypos, zpos)
{
	this.x = xpos;
	this.y = ypos;
	this.z = zpos;
	this.xVel = Math.random() - 0.5;
	this.yVel = Math.random() - 0.5;
	this.zVel = Math.random() - 0.5;
	var speed=Math.sqrt((this.xVel*this.xVel) + (this.yVel*this.yVel) + (this.zVel*this.zVel));
	this.xVel *= 40/speed;
	this.yVel *= 40/speed;
	this.zVel *= 40/speed;
	this.size = 1;
	this.domElement.style.visibility = "visible";
	this.enabled = true;
};

Particle.prototype.update = function()
{
	if (!this.enabled)
		return true;
	var drag=0.84;
	this.xVel *= drag;
	this.yVel *= drag;
	this.zVel *= drag;
	this.x += this.xVel;
	this.y += this.yVel;
	this.z += this.zVel;
	this.size *= 0.9;
	this.domElement.style.webkitTransform = "translate3d(" + this.x.toString() + "px, " + this.y.toString() + "px, " + this.z.toString() + "px) scale(" + this.size.toString() + ")";
	if (this.size<=0.05)
	{
		this.domElement.style.visibility = "hidden";
		this.enabled = false;
		return false;
	}
	return true;
};

// class Utils
var Utils = function() {};
Utils.typeName = 'Utils';

Utils.randomRange = function(min, max)
{
	return (Math.random()*(max - min)) + min;
};

function onLoad()
{
	initMouseListeners();
	window.setInterval(gameLoop, (1000/60)|0);
};

function gameLoop()
{
	if (counter++%20===0)
		makeFish();
	layer1.style.webkitTransform = "translate3d(0px, " + (-768 + ((counter*5)%768)).toString() + "px, -999px) scale(4)";
	layer2.style.webkitTransform = "translate3d(0px, " + (-768 + ((counter*10)%768)).toString() + "px, -998px) scale(4)";
	for (var i=0; i<fishes.length; i++)
	{
		var fish=fishes[i];
		if (!fish.update())
		{
			spareFishes.push(fish);
		}
	}
	emitter.update();
};

function makeFish()
{
	var fish;
	if (spareFishes.length>0)
	{
		fish = spareFishes.pop();
	}
	else
	{
		var fishImageURL="img/orangefish0" + ((fishes.length%4) + 1).toString() + ".png";
		fish = (new Fish().ctor$1(fishImageURL, 128, 128));
		fishes.push(fish);
		fish.domElement.addEventListener("mouseover", fishMouseOver, true);
		fish.domElement.addEventListener("touchstart", fishTouched, true);
		container.appendChild(fish.domElement);
	}
	fish.init(HALF_WIDTH + Utils.randomRange(-250, 250), SCREEN_HEIGHT + 100, Utils.randomRange(-250, 250));
};

function fishMouseOver(event)
{
	event.preventDefault();
	var fish=getFishFromElement(event.target);
	if (fish!==null)
		explodeFish(fish);
};

function fishTouched(event)
{
	event.preventDefault();
	for (var j=0; j<(event.changedTouches.length); j++)
	{
		var fish=getFishFromElement(event.target);
		if (fish!==null)
			explodeFish(fish);
	}
};

function getFishFromElement(domElement)
{
	for (var i=0; i<(fishes.length); i++)
	{
		if (fishes[i].domElement===domElement)
			return fishes[i];
	}
	return null;
};

function explodeFish(fish)
{
	playBurst();
	fish.explode(emitter);
	spareFishes.push(fish);
};

function initMouseListeners()
{
	document.addEventListener("mousemove", preventDefault, false);
	document.addEventListener("mousedown", preventDefault, false);
	document.addEventListener("mouseup", preventDefault, false);
	document.addEventListener("touchstart", preventDefault, false);
	document.addEventListener("touchmove", preventDefault, false);
	document.addEventListener("touchend", preventDefault, false);
};

function preventDefault(event)
{
	event.preventDefault();
};

function initSounds()
{
	burstSound = (new Audio("sounds/burstsml2.aif"));
	burstSound.load();
	burstSound.volume = 0;
	burstSound.play();
};

function playBurst()
{
	burstSound.volume = 1;
	burstSound.play();
};
// Call static constructors
Fish.$cctor();


// Start Global Code
var container=document.createElement("div");
var layer1=document.createElement("div");
var layer2=document.createElement("div");
var SCREEN_WIDTH=1024;
var SCREEN_HEIGHT=768;
var HALF_WIDTH=(SCREEN_WIDTH/2)|0;
var HALF_HEIGHT=(SCREEN_HEIGHT/2)|0;
var fishes=[];
var spareFishes=[];
var counter=0;
var burstSound=null;
var emitter=new Emitter().ctor$1(container);
initSounds();
layer1.className = "parallax";
layer1.style.background = "url(img/parallaxBack.jpg)";
document.body.appendChild(layer1);
layer2.className = "parallax";
layer2.style.background = "url(img/parallaxFront.png) transparent";
document.body.appendChild(layer2);
container.className = "container";
container.style.webkitPerspective = "400";
container.style.webkitPerspectiveOrigin = HALF_WIDTH.toString() + "px " + HALF_HEIGHT.toString() + "px";
container.style.width = SCREEN_WIDTH;
container.style.height = SCREEN_HEIGHT;
document.body.appendChild(container);
window.addEventListener("load", onLoad);

})();
